#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define PI 3.141592654
#define TORADIAN 0.017453
#define TOANGLE 57.29579

#ifndef NUM_CRYSTAL
#define NUM_CRYSTAL	10000
#endif

/* Crystal coordinate */
typedef struct _crystal_coordinate {
	/* for miller indices */
	double x1[3];
	double x2[3];
	double x3[3];

	/* for Euler angles */
	double pi1;
	double cpi;
	double pi2;
} c_coordinate;

void make_euler_crystal_coor(c_coordinate *crystal_coor){
	double pi1 = 0, cpi = 0, pi2 = 0;
	double h = 0, k = 0, l = 0;
	double u = 0, v = 0, w = 0;
	double a = 0, b = 0, c = 0;

	/* Get x2 components of Miller index */
	a = crystal_coor->x3[0];
	b = crystal_coor->x3[1];
	c = crystal_coor->x3[2];

	h = a / sqrt (a*a + b*b + c*c);
	k = b / sqrt (a*a + b*b + c*c);
	l = c / sqrt (a*a + b*b + c*c);

	a = crystal_coor->x1[0];
	b = crystal_coor->x1[1];
	c = crystal_coor->x1[2];

	u = a / sqrt (a*a + b*b + c*c);
	v = b / sqrt (a*a + b*b + c*c);
	w = c / sqrt (a*a + b*b + c*c);

	crystal_coor->x2[0] = l*v - k*w;
	crystal_coor->x2[1] = 0 - (l*u - h*w);
	crystal_coor->x2[2] = k*u - h*v;


	/* Get Euler angles */
	if (l == 1) {
		pi1 = atan(0 - v/u) * TOANGLE;
		cpi = 0;
		pi2 = 0;
	} else if (l == -1) {
		pi1 = atan(v/u) * TOANGLE;
		cpi = 180.0;
		pi2 = 0;
	} else {
		pi1 = asin((w/sqrt(u*u+v*v+w*w))*sqrt((h*h+k*k+l*l)/(h*h+k*k))) * TOANGLE;
		cpi = acos(l/sqrt(h*h+k*k+l*l)) * TOANGLE;
		pi2 = asin(h/sqrt(h*h+k*k)) * TOANGLE;
	}

	crystal_coor->pi1 = pi1;
	crystal_coor->cpi = cpi;
	crystal_coor->pi2 = pi2;
	return;
}

void make_miller_crystal_coor(c_coordinate *crystal_coor) {
	double pi1 = 0, cpi = 0, pi2 = 0;

	double t1 = 0, t2 = 0, t3 = 0;
	int it1 = 0, it2 = 0, it3 = 0, it4 = 0;

	pi1 = crystal_coor->pi1*TORADIAN;
	cpi = crystal_coor->cpi*TORADIAN;
	pi2 = crystal_coor->pi2*TORADIAN;

	/* for [100] */
	t1 = cos(pi1) * cos(pi2) - sin(pi1) * sin(pi2) * cos(cpi);
	t2 = 0 - cos(pi1) * sin(pi2) - sin(pi1) * cos(pi2) * cos(cpi);
	t3 = sin(pi1) * sin(cpi);

	crystal_coor->x1[0] = t1;
	crystal_coor->x1[1] = t2;
	crystal_coor->x1[2] = t3;
		
	/* for [010] */
	t1 = sin(pi1) * cos(pi2) + cos(pi1) * sin(pi2) * cos(cpi);
	t2 = 0 - sin(pi1) * sin(pi2) + cos(pi1) * cos(pi2) * cos(cpi);
	t3 = 0 - cos(pi1) * sin(cpi);

	crystal_coor->x2[0] = t1;
	crystal_coor->x2[1] = t2;
	crystal_coor->x2[2] = t3;

	/* for [001] */
	t1 = sin(pi2) * sin(cpi);
	t2 = cos(pi2) * sin(cpi);
	t3 = cos(cpi);

	crystal_coor->x3[0] = t1;
	crystal_coor->x3[1] = t2;
	crystal_coor->x3[2] = t3;

	return;
}

void get_nth_symmetry(c_coordinate *sym_crystal, c_coordinate crystal_coor, int nth) {
	double c[3][3] = {'\0'};

	if (nth == 0) {
		c[0][0] = 1.0, c[0][1] = 0.0, c[0][2] = 0.0;
		c[1][0] = 0.0, c[1][1] = 1.0, c[1][2] = 0.0;
		c[2][0] = 0.0, c[2][1] = 0.0, c[2][2] = 1.0;
	} else if (nth == 1) {
		c[0][0] = 0.0, c[0][1] = 0.0, c[0][2] = -1.0;
		c[1][0] = 0.0, c[1][1] = -1.0, c[1][2] = 0.0;
		c[2][0] = -1.0, c[2][1] = 0.0, c[2][2] = 0.0;
	} else if (nth == 2) {
		c[0][0] = 0.0, c[0][1] = 0.0, c[0][2] = -1.0;
		c[1][0] = 0.0, c[1][1] = 1.0, c[1][2] = 0.0;
		c[2][0] = 1.0, c[2][1] = 0.0, c[2][2] = 0.0;
	} else if (nth == 3) {
		c[0][0] = -1.0, c[0][1] = 0.0, c[0][2] = 0.0;
		c[1][0] = 0.0, c[1][1] = 1.0, c[1][2] = 0.0;
		c[2][0] = 0.0, c[2][1] = 0.0, c[2][2] = -1.0;
	} else if (nth == 4) {
		c[0][0] = 0.0, c[0][1] = 0.0, c[0][2] = 1.0;
		c[1][0] = 0.0, c[1][1] = 1.0, c[1][2] = 0.0;
		c[2][0] = -1.0, c[2][1] = 0.0, c[2][2] = 0.0;
	} else if (nth == 5) {
		c[0][0] = 1.0, c[0][1] = 0.0, c[0][2] = 0.0;
		c[1][0] = 0.0, c[1][1] = 0.0, c[1][2] = -1.0;
		c[2][0] = 0.0, c[2][1] = 1.0, c[2][2] = 0.0;
	} else if (nth == 6) {
		c[0][0] = 1.0, c[0][1] = 0.0, c[0][2] = 0.0;
		c[1][0] = 0.0, c[1][1] = -1.0, c[1][2] = 0.0;
		c[2][0] = 0.0, c[2][1] = 0.0, c[2][2] = -1.0;
	} else if (nth == 7) {
		c[0][0] = 1.0, c[0][1] = 0.0, c[0][2] = 0.0;
		c[1][0] = 0.0, c[1][1] = 0.0, c[1][2] = 1.0;
		c[2][0] = 0.0, c[2][1] = -1.0, c[2][2] = 0.0;
	} else if (nth == 8) {
		c[0][0] = 0.0, c[0][1] = -1.0, c[0][2] = 0.0;
		c[1][0] = 1.0, c[1][1] = 0.0, c[1][2] = 0.0;
		c[2][0] = 0.0, c[2][1] = 0.0, c[2][2] = 1.0;
	} else if (nth == 9) {
		c[0][0] = -1.0, c[0][1] = 0.0, c[0][2] = 0.0;
		c[1][0] = 0.0, c[1][1] = -1.0, c[1][2] = 0.0;
		c[2][0] = 0.0, c[2][1] = 0.0, c[2][2] = 1.0;
	} else if (nth == 10) {
		c[0][0] = 0.0, c[0][1] = 1.0, c[0][2] = 0.0;
		c[1][0] = -1.0, c[1][1] = 0.0, c[1][2] = 0.0;
		c[2][0] = 0.0, c[2][1] = 0.0, c[2][2] = 1.0;
	} else if (nth == 11) {
		c[0][0] = 0.0, c[0][1] = 0.0, c[0][2] = 1.0;
		c[1][0] = 1.0, c[1][1] = 0.0, c[1][2] = 0.0;
		c[2][0] = 0.0, c[2][1] = 1.0, c[2][2] = 0.0;
	} else if (nth == 12) {
		c[0][0] = 0.0, c[0][1] = 1.0, c[0][2] = 0.0;
		c[1][0] = 0.0, c[1][1] = 0.0, c[1][2] = 1.0;
		c[2][0] = 1.0, c[2][1] = 0.0, c[2][2] = 0.0;
	} else if (nth == 13) {
		c[0][0] = 0.0, c[0][1] = 0.0, c[0][2] = -1.0;
		c[1][0] = -1.0, c[1][1] = 0.0, c[1][2] = 0.0;
		c[2][0] = 0.0, c[2][1] = 1.0, c[2][2] = 0.0;
	} else if (nth == 14) {
		c[0][0] = 0.0, c[0][1] = -1.0, c[0][2] = 0.0;
		c[1][0] = 0.0, c[1][1] = 0.0, c[1][2] = 1.0;
		c[2][0] = -1.0, c[2][1] = 0.0, c[2][2] = 0.0;
	} else if (nth == 15) {
		c[0][0] = 0.0, c[0][1] = 1.0, c[0][2] = 0.0;
		c[1][0] = 0.0, c[1][1] = 0.0, c[1][2] = -1.0;
		c[2][0] = -1.0, c[2][1] = 0.0, c[2][2] = 0.0;
	} else if (nth == 16) {
		c[0][0] = 0.0, c[0][1] = 0.0, c[0][2] = -1.0;
		c[1][0] = 1.0, c[1][1] = 0.0, c[1][2] = 0.0;
		c[2][0] = 0.0, c[2][1] = -1.0, c[2][2] = 0.0;
	} else if (nth == 17) {
		c[0][0] = 0.0, c[0][1] = 0.0, c[0][2] = 1.0;
		c[1][0] = -1.0, c[1][1] = 0.0, c[1][2] = 0.0;
		c[2][0] = 0.0, c[2][1] = -1.0, c[2][2] = 0.0;
	} else if (nth == 18) {
		c[0][0] = 0.0, c[0][1] = -1.0, c[0][2] = 0.0;
		c[1][0] = 0.0, c[1][1] = 0.0, c[1][2] = -1.0;
		c[2][0] = 1.0, c[2][1] = 0.0, c[2][2] = 0.0;
	} else if (nth == 19) {
		c[0][0] = 0.0, c[0][1] = 1.0, c[0][2] = 0.0;
		c[1][0] = 1.0, c[1][1] = 0.0, c[1][2] = 0.0;
		c[2][0] = 0.0, c[2][1] = 0.0, c[2][2] = -1.0;
	} else if (nth == 20) {
		c[0][0] = -1.0, c[0][1] = 0.0, c[0][2] = 0.0;
		c[1][0] = 0.0, c[1][1] = 0.0, c[1][2] = 1.0;
		c[2][0] = 0.0, c[2][1] = 1.0, c[2][2] = 0.0;
	} else if (nth == 21) {
		c[0][0] = 0.0, c[0][1] = 0.0, c[0][2] = 1.0;
		c[1][0] = 0.0, c[1][1] = -1.0, c[1][2] = 0.0;
		c[2][0] = 1.0, c[2][1] = 0.0, c[2][2] = 0.0;
	} else if (nth == 22) {
		c[0][0] = 0.0, c[0][1] = -1.0, c[0][2] = 0.0;
		c[1][0] = -1.0, c[1][1] = 0.0, c[1][2] = 0.0;
		c[2][0] = 0.0, c[2][1] = 0.0, c[2][2] = -1.0;
	} else if (nth == 23) {
		c[0][0] = -1.0, c[0][1] = 0.0, c[0][2] = 0.0;
		c[1][0] = 0.0, c[1][1] = 0.0, c[1][2] = -1.0;
		c[2][0] = 0.0, c[2][1] = -1.0, c[2][2] = 0.0;
	} else {
		c[0][0] = 1.0, c[0][1] = 0.0, c[0][2] = 0.0;
		c[1][0] = 0.0, c[1][1] = 1.0, c[1][2] = 0.0;
		c[2][0] = 0.0, c[2][1] = 0.0, c[2][2] = 1.0;
	}
	
	sym_crystal->x1[0] = c[0][0] * crystal_coor.x1[0]
				+ c[0][1] * crystal_coor.x1[1]
				+ c[0][2] * crystal_coor.x1[2];
	
	sym_crystal->x1[1] = c[1][0] * crystal_coor.x1[0]
				+ c[1][1] * crystal_coor.x1[1]
				+ c[1][2] * crystal_coor.x1[2];

	sym_crystal->x1[2] = c[2][0] * crystal_coor.x1[0]
				+ c[2][1] * crystal_coor.x1[1]
				+ c[2][2] * crystal_coor.x1[2];


	sym_crystal->x2[0] = c[0][0] * crystal_coor.x2[0]
				+ c[0][1] * crystal_coor.x2[1]
				+ c[0][2] * crystal_coor.x2[2];
	
	sym_crystal->x2[1] = c[1][0] * crystal_coor.x2[0]
				+ c[1][1] * crystal_coor.x2[1]
				+ c[1][2] * crystal_coor.x2[2];

	sym_crystal->x2[2] = c[2][0] * crystal_coor.x2[0]
				+ c[2][1] * crystal_coor.x2[1]
				+ c[2][2] * crystal_coor.x2[2];


	sym_crystal->x3[0] = c[0][0] * crystal_coor.x3[0]
				+ c[0][1] * crystal_coor.x3[1]
				+ c[0][2] * crystal_coor.x3[2];
	
	sym_crystal->x3[1] = c[1][0] * crystal_coor.x3[0]
				+ c[1][1] * crystal_coor.x3[1]
				+ c[1][2] * crystal_coor.x3[2];

	sym_crystal->x3[2] = c[2][0] * crystal_coor.x3[0]
				+ c[2][1] * crystal_coor.x3[1]
				+ c[2][2] * crystal_coor.x3[2];

	make_euler_crystal_coor(sym_crystal);
	return;
}

double get_misorientation(c_coordinate A, c_coordinate B) {
	double a, b, c, d, e, f, g, h, i;
	c_coordinate result, B_inv;
	double determinant = 0.0;
	double ret = 0.0, u1 = 0.0, u2 = 0.0, u3 = 0.0;

	a = B.x1[0], b = B.x2[0], c = B.x3[0];
	d = B.x1[1], e = B.x2[1], f = B.x3[1];
	g = B.x1[2], h = B.x2[2], i = B.x3[2];

	determinant = a*e*i + b*f*g + c*d*h - c*e*g - b*d*i - a*f*h;

	if (determinant == 0) return 360;

#ifdef __U_DEBUG__
	printf("Determinant: %f\n", determinant);
#endif

	B_inv.x1[0] = (1.0/determinant) * (e*i - f*h);
	B_inv.x1[1] = (1.0/determinant) * (f*g - d*i);
	B_inv.x1[2] = (1.0/determinant) * (d*h - e*g);
	B_inv.x2[0] = (1.0/determinant) * (c*h - b*i);
	B_inv.x2[1] = (1.0/determinant) * (a*i - c*g);
	B_inv.x2[2] = (1.0/determinant) * (b*g - a*h);
	B_inv.x3[0] = (1.0/determinant) * (b*f - c*e);
	B_inv.x3[1] = (1.0/determinant) * (c*d - a*f);
	B_inv.x3[2] = (1.0/determinant) * (a*e - b*d);

	result.x1[0] = A.x1[0]*B_inv.x1[0] + A.x2[0]*B_inv.x1[1] + A.x3[0] * B_inv.x1[2];
	result.x1[1] = A.x1[1]*B_inv.x1[0] + A.x2[1]*B_inv.x1[1] + A.x3[1] * B_inv.x1[2];
	result.x1[2] = A.x1[2]*B_inv.x1[0] + A.x2[2]*B_inv.x1[1] + A.x3[2] * B_inv.x1[2];

	result.x2[0] = A.x1[0]*B_inv.x2[0] + A.x2[0]*B_inv.x2[1] + A.x3[0] * B_inv.x2[2];
	result.x2[1] = A.x1[1]*B_inv.x2[0] + A.x2[1]*B_inv.x2[1] + A.x3[1] * B_inv.x2[2];
	result.x2[2] = A.x1[2]*B_inv.x2[0] + A.x2[2]*B_inv.x2[1] + A.x3[2] * B_inv.x2[2];

	result.x3[0] = A.x1[0]*B_inv.x3[0] + A.x2[0]*B_inv.x3[1] + A.x3[0] * B_inv.x3[2];
	result.x3[1] = A.x1[1]*B_inv.x3[0] + A.x2[1]*B_inv.x3[1] + A.x3[1] * B_inv.x3[2];
	result.x3[2] = A.x1[2]*B_inv.x3[0] + A.x2[2]*B_inv.x3[1] + A.x3[2] * B_inv.x3[2];

	ret = acos(((result.x1[0] + result.x2[1] + result.x3[2] - 1.0) / 2.0)) * TOANGLE;	

	u1 = (result.x3[1] - result.x2[2]);
	u2 = (result.x1[2] - result.x3[0]);
	u3 = (result.x2[0] - result.x1[1]);

#ifdef __U_DEBUG__
	printf("Product matrix\n");
	printf("x1 : %f %f %f\n",
			result.x1[0],
			result.x1[1],
			result.x1[2]);
	printf("x2 : %f %f %f\n",
			result.x2[0],
			result.x2[1],
			result.x2[2]);
	printf("x3 : %f %f %f\n",
			result.x3[0],
			result.x3[1],
			result.x3[2]);
#endif


#ifdef __U_DEBUG__
	printf("Miss orientation: %f\n", ret);
	printf("Axis %f, %f %f\n", u1, u2 ,u3);
#endif

	/* TEST the axis is on FZ */
	if (u1 >= 0 && u2 >= 0 && u3 >= 0) {
		return ret;
	}  
	return 360; 
}

int main (int argc, char ** argv) {
	int i = 0, j = 0, k = 0;
	int nRet = 0;
	float tempf = 0;

	c_coordinate crystal_coor[NUM_CRYSTAL] = {'\0'};
	c_coordinate target_coor;
	double tolerance_angle = 0;

	c_coordinate sym_crystal, sym_target_crystal;
	double temp_misorientation = 0, misorientation = 0;

	int count = 0;

	FILE * fp = NULL;
	int l = 0, plot[360] = {'\0'};

	/* Now, Make the grains */
	/* give the random orientations using Euler angles {0~359, 0~359, 0~359} */
	/* 1. get the 3 random integers (angles) (0~359) for each grains */
	/* 2. Euler angles -> miller indices */

	system("clear");
	printf("WELCOME!\n");

#ifdef __U_DEBUG__
	printf("Randomly generated crystals (Euler anlges)\n");
#endif

	/* 1. get the 3 angles for each grains */
	srand(time(0));

	for(i = 0; i < NUM_CRYSTAL; i++) {
		crystal_coor[i].pi1 = (rand() % 36000)/100.0;
		crystal_coor[i].cpi = (rand() % 36000)/100.0;
		crystal_coor[i].pi2 = (rand() % 36000)/100.0;

#ifdef __U_DEBUG__
	printf("Crystal # : %d\n", i+1);
	printf("Euler angles : %f %f %f\n",
		crystal_coor[i].pi1,
		crystal_coor[i].cpi,
		crystal_coor[i].pi2);
#endif
	}

	/* 2. Euler angles -> miller indices */
	/* matrix for Euler rotations
	[	cos(pi1)cos(pi2) - sin(pi1)sin(pi2)cos(cpi)	sin(pi1)cos(pi2) + cos(pi1)sin(pi2)cos(cpi)	sin(pi2)sin(cpi)	]
	[	-cos(pi1)sin(pi2) - sin(pi1)cos(pi2)cos(cpi)	-sin(pi1)sin(pi2) + cos(pi1)cos(pi2)cos(cpi)	cos(pi2)sin(cpi)	]
	[	sin(pi1)sin(cpi)				-cos(pi1)sin(cpi)				cos(cpi)		]
	*/

#ifdef __U_DEBUG__
	printf("Randomly generated crystals (Miller indices)\n");
#endif
	for(i = 0; i < NUM_CRYSTAL; i++) {
		make_miller_crystal_coor(&crystal_coor[i]);
#ifdef __U_DEBUG__
	printf("Crystal # : %d\n", i+1);
	printf("x1 : %.6lf %.6lf %.6lf\n",
			crystal_coor[i].x1[0],
			crystal_coor[i].x1[1],
			crystal_coor[i].x1[2]);
	printf("x2 : %.6lf %.6lf %.6lf\n",
			crystal_coor[i].x2[0],
			crystal_coor[i].x2[1],
			crystal_coor[i].x2[2]);
	printf("x3 : %.6lf %.6lf %.6lf\n",
			crystal_coor[i].x3[0],
			crystal_coor[i].x3[1],
			crystal_coor[i].x3[2]);
#endif
	}

	/* find the grains which have {100}<001> texture components */
	/* The degree of tolerance is 11.25 */
	printf("Please input the Euler angles to investigate.\n");
	printf("pi1: ");
	if(scanf("%f", &tempf)) {
		printf("The input value was: %f\n", tempf);
		target_coor.pi1 = tempf;
	} else {
		fprintf(stderr, "Error reading the value from user\n");
		goto err;
	}

	printf("cpi: ");
	if(scanf("%f", &tempf)) {
		printf("The input value was: %f\n", tempf);
		target_coor.cpi = tempf;
	} else {
		fprintf(stderr, "Error reading the value from user\n");
		goto err;
	}

	printf("pi2: ");
	if(scanf("%f", &tempf)) {
		printf("The input value was: %f\n", tempf);
		target_coor.pi2 = tempf;
	} else {
		fprintf(stderr, "Error reading the value from user\n");
		goto err;
	}

	printf("Please input the tolerance angle for the Euler angles.\n");
	printf("Tolerance angle: ");
	if(scanf("%f", &tempf)) {
		printf("The input value was: %f\n", tempf);
		tolerance_angle = tempf;
	} else {
		fprintf(stderr, "Error reading the value from user\n");
		goto err;
	}

	make_miller_crystal_coor(&target_coor);

#ifdef __U_INFO__
	printf("Target Crystal, tolerance_angle : %lf\n", tolerance_angle);
	printf("x1 : %.6lf %.6lf %.6lf\n",
			target_coor.x1[0],
			target_coor.x1[1],
			target_coor.x1[2]);
	printf("x2 : %.6lf %.6lf %.6lf\n",
			target_coor.x2[0],
			target_coor.x2[1],
			target_coor.x2[2]);
	printf("x3 : %.6lf %.6lf %.6lf\n",
			target_coor.x3[0],
			target_coor.x3[1],
			target_coor.x3[2]);
	printf("pi1, cpi, pi2: %.6lf %.6lf %.6lf\n",
			target_coor.pi1,
			target_coor.cpi,
			target_coor.pi2);
#endif

	if (argc == 1) {
	i = j = k = count = 0;
	for(i=0; i < NUM_CRYSTAL; i++) {
		misorientation = 360; /* Maximum misorientation */
		for(j=0; j < 24; j++) {
			/* get nth symmetry for crystal */
			get_nth_symmetry(&sym_crystal, crystal_coor[i], j);
			/* get nth symmetry for target */
			for(k=0; k < 24; k++) {
				get_nth_symmetry(&sym_target_crystal, target_coor, k);
				temp_misorientation =
					get_misorientation(sym_target_crystal, sym_crystal);
			
#ifdef __U_DEBUG__
					printf("!!!! %dth crystal - %dth symmetry with %dthe target symmetry!!!!\n", i, j, k);
					printf("misorientation : %f, min : %f\n", temp_misorientation, misorientation);
#endif

				if(temp_misorientation <= misorientation) misorientation = temp_misorientation;
			memset(&sym_target_crystal, 0x00, sizeof(c_coordinate));
			}
			memset(&sym_crystal, 0x00, sizeof(c_coordinate));
		}
		
		if (misorientation <= tolerance_angle) count++;
	}

	printf("Total number of grains : %d\n", NUM_CRYSTAL);
	printf("Total number of grains within misorientation : %d\n", count);
	printf("Fraction : %lf\n", (double)count/(double)NUM_CRYSTAL);

	} else {
	/* Mackenzie plot */
	i = j = k = count = 0;
	for(i=0; i < NUM_CRYSTAL; i++) {
		for (l=i+1; l < NUM_CRYSTAL; l++) {
		memcpy(&target_coor, &crystal_coor[l], sizeof(c_coordinate));
		misorientation = 360; /* Maximum misorientation */
		for(j=0; j < 24; j++) {
			/* get nth symmetry for crystal */
			get_nth_symmetry(&sym_crystal, crystal_coor[i], j);
			/* get nth symmetry for target */
			for(k=0; k < 24; k++) {
				get_nth_symmetry(&sym_target_crystal, target_coor, k);
				temp_misorientation =
					get_misorientation(sym_target_crystal, sym_crystal);
			
#ifdef __U_DEBUG__
					printf("!!!! %dth crystal - %dth symmetry with %dthe target symmetry!!!!\n", i, j, k);
					printf("misorientation : %f, min : %f\n", temp_misorientation, misorientation);
#endif

				if(temp_misorientation <= misorientation) misorientation = temp_misorientation;
			memset(&sym_target_crystal, 0x00, sizeof(c_coordinate));
			}
			memset(&sym_crystal, 0x00, sizeof(c_coordinate));
		}
		plot[(int)misorientation]++;
		}
		
		if (misorientation <= tolerance_angle) count++;
	}

	printf("Total number of grains : %d\n", NUM_CRYSTAL);
	printf("Total number of grains within misorientation : %d\n", count);
	printf("Fraction : %lf\n", (double)count/(double)NUM_CRYSTAL);

	fp = fopen("./random.out", "w+");
	fprintf(fp, "Angle of disorientation : Count\n");
	for (i=0; i<360; i++) {
		fprintf(fp, "%d : %d\n", i, plot[i]);
	}

	fclose(fp);
	printf("\nrandom.out file was created.\n");
	}
	return 1;
err:
	return -1;
}

